@@ -14,7 +14,7 @@
  */
 
 #include "Python.h"
-#if PY_MAJOR_VERSION < 3
+#if PY_MAJOR_VERSION < 3 && PY_MINOR_VERSION < 6
 #include "stringobject.h"
 #define PyUnicode_FromString PyString_FromString
 #endif
@@ -183,16 +183,23 @@ static int alsapcm_setup(alsapcm_t *self)
     snd_pcm_hw_params_set_access(self->handle, hwparams, 
                                  SND_PCM_ACCESS_RW_INTERLEAVED);
     snd_pcm_hw_params_set_format(self->handle, hwparams, self->format);
-    snd_pcm_hw_params_set_channels(self->handle, hwparams, self->channels);
+    res = snd_pcm_hw_params_set_channels(self->handle, hwparams, 
+                                         self->channels);
+    if (res < 0) return res;
+
     dir = 0;
-    snd_pcm_hw_params_set_rate(self->handle, hwparams, self->rate, dir);
-    snd_pcm_hw_params_set_period_size(self->handle, hwparams, 
+    res = snd_pcm_hw_params_set_rate(self->handle, hwparams, self->rate, dir);
+    if (res < 0) return res;
+    
+    res = snd_pcm_hw_params_set_period_size(self->handle, hwparams, 
                                       self->periodsize, dir);
-    snd_pcm_hw_params_set_periods(self->handle, hwparams, 4, 0);
+    if (res < 0) return res;
+
+    res = snd_pcm_hw_params_set_periods(self->handle, hwparams, 4, 0);
+    if (res < 0) return res;
     
     /* Write it to the device */
     res = snd_pcm_hw_params(self->handle, hwparams);
-    if (res) return res;
     
     /* Query current settings. These may differ from the requested values,
        which should therefore be sync'ed with actual values */
@@ -1559,7 +1566,7 @@ alsamixer_getrec(alsamixer_t *self, PyObject *args)
         if (snd_mixer_selem_has_capture_channel(elem, i)) 
         {
             snd_mixer_selem_get_capture_switch(elem, i, &ival);
-            PyList_Append(result,PyLong_FromLong(!ival));
+            PyList_Append(result,PyLong_FromLong(ival));
         }
     }
     return result;
@@ -1752,7 +1759,7 @@ alsamixer_setrec(alsamixer_t *self, PyObject *args)
         {
             if (snd_mixer_selem_has_capture_channel(elem, i)) 
             {
-                snd_mixer_selem_set_playback_switch(elem, i, rec);
+                snd_mixer_selem_set_capture_switch(elem, i, rec);
                 done++;
             }
         }
